@@ -34,7 +34,7 @@ from ..opgp import OpgpController, KEY_SLOT, TOUCH_MODE
 from ..driver_ccid import APDUError, SW
 from .util import (
     click_force_option, click_format_option, click_postpone_execution,
-    UpperCaseChoice)
+    EnumChoice)
 
 
 logger = logging.getLogger(__name__)
@@ -100,7 +100,9 @@ def info(ctx):
     Display status of OpenPGP application.
     """
     controller = ctx.obj['controller']
-    click.echo('OpenPGP version: %d.%d.%d' % controller.version)
+    click.echo('OpenPGP version: %d.%d' % controller.get_openpgp_version())
+    click.echo('Application version: %d.%d.%d' % controller.version)
+    click.echo()
     retries = controller.get_remaining_pin_tries()
     click.echo('PIN tries remaining: {}'.format(retries.pin))
     click.echo('Reset code tries remaining: {}'.format(retries.reset))
@@ -111,17 +113,17 @@ def info(ctx):
         click.echo('Touch policies')
         click.echo(
             'Signature key           {!s}'.format(
-                controller.get_touch(KEY_SLOT.SIGNATURE)))
+                controller.get_touch(KEY_SLOT.SIG)))
         click.echo(
             'Encryption key          {!s}'.format(
-                controller.get_touch(KEY_SLOT.ENCRYPTION)))
+                controller.get_touch(KEY_SLOT.ENC)))
         click.echo(
             'Authentication key      {!s}'.format(
-                controller.get_touch(KEY_SLOT.AUTHENTICATION)))
+                controller.get_touch(KEY_SLOT.AUT)))
         if controller.supports_attestation:
             click.echo(
                 'Attestation key         {!s}'.format(
-                    controller.get_touch(KEY_SLOT.ATTESTATION)))
+                    controller.get_touch(KEY_SLOT.ATT)))
 
 
 @openpgp.command()
@@ -149,13 +151,8 @@ def echo_default_pins():
 
 
 @openpgp.command('set-touch')
-@click.argument(
-    'key', metavar='KEY', type=UpperCaseChoice(['AUT', 'ENC', 'SIG', 'ATT']),
-    callback=lambda c, p, v: KEY_SLOT(v))
-@click.argument(
-    'policy', metavar='POLICY',
-    type=UpperCaseChoice(['ON', 'OFF', 'FIXED', 'CACHED', 'CACHED-FIXED']),
-    callback=lambda c, p, v: TOUCH_MODE[v.replace('-', '_')])
+@click.argument('key', metavar='KEY', type=EnumChoice(KEY_SLOT))
+@click.argument('policy', metavar='POLICY', type=EnumChoice(TOUCH_MODE))
 @click.option('-a', '--admin-pin', help='Admin PIN for OpenPGP.')
 @click_force_option
 @click.pass_context
@@ -179,11 +176,12 @@ def set_touch(ctx, key, policy, admin_pin, force):
 
     if force or click.confirm(
         'Set touch policy of {} key to {}?'.format(
-            key.name.lower(),
+            key.value.lower(),
             policy_name),
             abort=True, err=True):
         try:
-            controller.set_touch(key, policy, admin_pin)
+            controller.verify_admin(admin_pin)
+            controller.set_touch(key, policy)
         except APDUError as e:
             if e.sw == SW.SECURITY_CONDITION_NOT_SATISFIED:
                 ctx.fail('Touch policy not allowed.')
@@ -223,8 +221,9 @@ def set_pin_retries(
                 pin_retries, reset_code_retries,
                 admin_pin_retries), abort=True, err=True):
 
+        controller.verify_admin(admin_pin)
         controller.set_pin_retries(
-            pin_retries, reset_code_retries, admin_pin_retries, admin_pin)
+            pin_retries, reset_code_retries, admin_pin_retries)
 
         if resets_pins:
             click.echo('Default PINs are set.')
@@ -235,9 +234,7 @@ def set_pin_retries(
 @click.pass_context
 @click.option('-P', '--pin', help='PIN code.')
 @click_format_option
-@click.argument(
-    'key', metavar='KEY', type=UpperCaseChoice(['AUT', 'ENC', 'SIG']),
-    callback=lambda c, p, v: KEY_SLOT(v))
+@click.argument('key', metavar='KEY', type=EnumChoice(KEY_SLOT))
 @click.argument('certificate', type=click.File('wb'), metavar='CERTIFICATE')
 def attest(ctx, key, certificate, pin, format):
     """
@@ -265,12 +262,13 @@ def attest(ctx, key, certificate, pin, format):
 
     if not cert or click.confirm(
             'There is already data stored in the certificate slot for {}, '
-            'do you want to overwrite it?'.format(key.name)):
-        touch_policy = controller.get_touch(KEY_SLOT.ATTESTATION)
+            'do you want to overwrite it?'.format(key.value)):
+        touch_policy = controller.get_touch(KEY_SLOT.ATT)
         if touch_policy in [TOUCH_MODE.ON, TOUCH_MODE.FIXED]:
             click.echo('Touch your YubiKey...')
         try:
-            cert = controller.attest(key, pin)
+            controller.verify_pin(pin)
+            cert = controller.attest(key)
             certificate.write(cert.public_bytes(encoding=format))
         except Exception as e:
             logger.debug('Failed to attest', exc_info=e)
@@ -279,14 +277,12 @@ def attest(ctx, key, certificate, pin, format):
 
 @openpgp.command('export-certificate')
 @click.pass_context
-@click.argument(
-    'key', metavar='KEY', type=UpperCaseChoice(['AUT', 'ENC', 'SIG', 'ATT']),
-    callback=lambda c, p, v: KEY_SLOT(v))
+@click.argument('key', metavar='KEY', type=EnumChoice(KEY_SLOT))
 @click_format_option
 @click.argument('certificate', type=click.File('wb'), metavar='CERTIFICATE')
 def export_certificate(ctx, key, format, certificate):
     """
-    Export an OpenPGP Cardholder certificate.
+    Export an OpenPGP certificate.
 
     \b
     KEY         Key slot to read from (sig, enc, aut, or att).
@@ -303,12 +299,10 @@ def export_certificate(ctx, key, format, certificate):
 @openpgp.command('delete-certificate')
 @click.option('-a', '--admin-pin', help='Admin PIN for OpenPGP.')
 @click.pass_context
-@click.argument(
-    'key', metavar='KEY', type=UpperCaseChoice(['AUT', 'ENC', 'SIG', 'ATT']),
-    callback=lambda c, p, v: KEY_SLOT(v))
+@click.argument('key', metavar='KEY', type=EnumChoice(KEY_SLOT))
 def delete_certificate(ctx, key, admin_pin):
     """
-    Delete an OpenPGP Cardholder certificate.
+    Delete an OpenPGP certificate.
 
     \b
     KEY         Key slot to delete certificate from (sig, enc, aut, or att).
@@ -317,7 +311,8 @@ def delete_certificate(ctx, key, admin_pin):
     if admin_pin is None:
         admin_pin = click.prompt('Enter admin PIN', hide_input=True, err=True)
     try:
-        controller.delete_certificate(key, admin_pin)
+        controller.verify_admin(admin_pin)
+        controller.delete_certificate(key)
     except Exception as e:
         logger.debug('Failed to delete ', exc_info=e)
         ctx.fail('Failed to delete certificate.')
@@ -326,13 +321,11 @@ def delete_certificate(ctx, key, admin_pin):
 @openpgp.command('import-certificate')
 @click.option('-a', '--admin-pin', help='Admin PIN for OpenPGP.')
 @click.pass_context
-@click.argument(
-    'key', metavar='KEY', type=UpperCaseChoice(['AUT', 'ENC', 'SIG', 'ATT']),
-    callback=lambda c, p, v: KEY_SLOT(v))
+@click.argument('key', metavar='KEY', type=EnumChoice(KEY_SLOT))
 @click.argument('cert', type=click.File('rb'), metavar='CERTIFICATE')
 def import_certificate(ctx, key, cert, admin_pin):
     """
-    Import an OpenPGP Cardholder certificate.
+    Import an OpenPGP certificate.
 
     \b
     KEY         Key slot to import certificate to (sig, enc, aut, or att).
@@ -351,7 +344,8 @@ def import_certificate(ctx, key, cert, admin_pin):
     if len(certs) != 1:
         ctx.fail('Can only import one certificate.')
     try:
-        controller.import_certificate(key, certs[0], admin_pin)
+        controller.verify_admin(admin_pin)
+        controller.import_certificate(key, certs[0])
     except Exception as e:
         logger.debug('Failed to import', exc_info=e)
         ctx.fail('Failed to import certificate')
@@ -380,7 +374,8 @@ def import_attestation_key(ctx, private_key, admin_pin):
         logger.debug('Failed to parse', exc_info=e)
         ctx.fail('Failed to parse private key.')
     try:
-        controller.import_attestation_key(private_key, admin_pin)
+        controller.verify_admin(admin_pin)
+        controller.import_key(KEY_SLOT.ATT, private_key)
     except Exception as e:
         logger.debug('Failed to import', exc_info=e)
         ctx.fail('Failed to import attestation key.')
