@@ -32,17 +32,18 @@ import click
 import sys
 from ..util import parse_b32_key
 from collections import OrderedDict, MutableMapping
-
-click_force_option = click.option('-f', '--force', is_flag=True,
-                                  help='Confirm the action without prompting.')
+from cryptography.hazmat.primitives import serialization
 
 
 class UpperCaseChoice(click.Choice):
     """
     Support lowercase option values for uppercase options.
     Does not support token normalization.
+    Choice options MUST be all uppercase.
     """
     def __init__(self, choices):
+        for v in choices:
+            assert v.upper() == v, 'Choice not all uppercase: ' + v
         click.Choice.__init__(self, choices)
 
     def convert(self, value, param, ctx):
@@ -53,6 +54,24 @@ class UpperCaseChoice(click.Choice):
                 value, ', '.join(self.choices)), param, ctx)
 
 
+class EnumChoice(UpperCaseChoice):
+    """
+    Use an enum's member names as the definition for a choice option.
+
+    Enum member names MUST be all uppercase. Options are not case sensitive.
+    Underscores in enum names are translated to dashes in the option choice.
+    """
+    def __init__(self, choices_enum):
+        super(EnumChoice, self).__init__(
+            [v.name.replace('_', '-') for v in choices_enum])
+        self.choices_enum = choices_enum
+
+    def convert(self, value, param, ctx):
+        name = super(EnumChoice, self).convert(
+            value.replace('-', '_'), param, ctx)
+        return self.choices_enum[name]
+
+
 def click_callback(invoke_on_missing=False):
     def wrap(f):
         @functools.wraps(f)
@@ -68,6 +87,26 @@ def click_callback(invoke_on_missing=False):
     return wrap
 
 
+@click_callback()
+def click_parse_format(ctx, param, val):
+    if val == 'PEM':
+        return serialization.Encoding.PEM
+    elif val == 'DER':
+        return serialization.Encoding.DER
+    else:
+        raise ValueError(val)
+
+
+click_force_option = click.option(
+    '-f', '--force', is_flag=True, help='Confirm the action without prompting.')
+
+
+click_format_option = click.option(
+    '-F', '--format',
+    type=UpperCaseChoice(['PEM', 'DER']), default='PEM', show_default=True,
+    help='Encoding format.', callback=click_parse_format)
+
+
 class YkmanContextObject(MutableMapping):
     def __init__(self):
         self._objects = OrderedDict()
@@ -81,7 +120,7 @@ class YkmanContextObject(MutableMapping):
     def resolve(self):
         if not self._resolved:
             self._resolved = True
-            for k, f in self._objects.items():
+            for k, f in self._objects.copy().items():
                 self._objects[k] = f()
 
     def __getitem__(self, key):
